@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -24,6 +24,9 @@
 #include "folder.h"
 #include "file.h"
 #include "entry.h"
+#include "stream.h"
+#include "pod/data.h"
+#include "pod/map.h"
 #include <mobius/exception.inc>
 #include <stdexcept>
 
@@ -118,6 +121,58 @@ tp_getter_short_name (io_folder_o *self)
   return ret;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief path getter
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_getter_path (io_folder_o *self)
+{
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pystring_from_std_string (self->obj->get_path ());
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief path setter
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static int 
+tp_setter_path (io_folder_o *self, PyObject *value, void *)
+{ 
+  if (value == nullptr)
+    { 
+      PyErr_SetString (PyExc_TypeError, "cannot delete 'path' attribute");
+      return -1;
+    }
+
+  if (!mobius::py::pystring_check (value))
+    {
+      PyErr_SetString (PyExc_TypeError, "invalid type for 'path' attribute");
+      return -1;
+    }
+
+  // set path
+  try
+    {
+      self->obj->set_path (mobius::py::pystring_as_std_string (value));
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+      return -1;
+    }
+
+  return 0;
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief inode getter
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -138,6 +193,26 @@ tp_getter_inode (io_folder_o *self)
   return ret;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief size getter
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_getter_size (io_folder_o *self)
+{
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pylong_from_std_uint64_t (self->obj->get_size ());
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  return ret;
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief user_id getter
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -375,12 +450,24 @@ static PyGetSetDef tp_getset[] =
     (setter) 0,
     (char *) "short file name", NULL
   },
+  {
+    (char *) "path",
+    (getter) tp_getter_path,
+    (setter) tp_setter_path,
+    (char *) "path", NULL
+  },
   {
     (char *) "inode",
     (getter) tp_getter_inode,
     (setter) 0,
     (char *) "inode", NULL
   },
+  {
+    (char *) "size",
+    (getter) tp_getter_size,
+    (setter) 0,
+    (char *) "folder size in bytes", NULL
+  },
   {
     (char *) "user_id",
     (getter) tp_getter_user_id,
@@ -550,6 +637,46 @@ tp_f_is_browseable (io_folder_o *self)
   return ret;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief is_folder method
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_is_folder (io_folder_o *)
+{
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pybool_from_bool (true);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief is_file method
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_is_file (io_folder_o *)
+{
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pybool_from_bool (false);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  return ret;
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief get_extension method
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -594,7 +721,7 @@ tp_f_get_parent (io_folder_o *self, PyObject *)
 //! \brief get_children method
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 static PyObject *
-tp_f_get_children (io_folder_o *self)
+tp_f_get_children (io_folder_o *self, PyObject *)
 {
   PyObject *ret = nullptr;
 
@@ -613,6 +740,121 @@ tp_f_get_children (io_folder_o *self)
   return ret;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief get_child_by_name method
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_get_child_by_name (io_folder_o *self, PyObject *args)
+{
+  // parse input args
+  std::string arg_name;
+  bool arg_case_sensitive;
+
+  try
+    {
+      arg_name = mobius::py::get_arg_as_std_string (args, 0);
+      arg_case_sensitive = mobius::py::get_arg_as_bool (args, 1, true);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
+
+  // execute C++ code
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = PyMobius_IO_Entry_from_cpp (
+              self->obj->get_child_by_name (arg_name, arg_case_sensitive)
+            );
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief get_child_by_path method
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_get_child_by_path (io_folder_o *self, PyObject *args)
+{
+  // parse input args
+  std::string arg_name;
+  bool arg_case_sensitive;
+
+  try
+    {
+      arg_name = mobius::py::get_arg_as_std_string (args, 0);
+      arg_case_sensitive = mobius::py::get_arg_as_bool (args, 1, true);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
+
+  // execute C++ code
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = PyMobius_IO_Entry_from_cpp (
+              self->obj->get_child_by_path (arg_name, arg_case_sensitive)
+            );
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief get_children_by_name method
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_get_children_by_name (io_folder_o *self, PyObject *args)
+{
+  // parse input args
+  std::string arg_name;
+  bool arg_case_sensitive;
+
+  try
+    {
+      arg_name = mobius::py::get_arg_as_std_string (args, 0);
+      arg_case_sensitive = mobius::py::get_arg_as_bool (args, 1, true);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
+
+  // execute C++ code
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pylist_from_cpp_container (
+               self->obj->get_children_by_name (arg_name, arg_case_sensitive),
+               PyMobius_IO_Entry_from_cpp
+            );
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  return ret;
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief new_file method
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -887,6 +1129,317 @@ tp_f_rename (io_folder_o *self, PyObject *args)
   return ret;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief set_tag method
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_set_tag (io_folder_o *self, PyObject *args)
+{
+  // parse input args
+  std::string arg_tag;
+
+  try
+    {
+      arg_tag = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
+
+  // execute C++ code
+  PyObject *ret = nullptr;
+
+  try
+    {
+      self->obj->set_tag (arg_tag);
+      ret = mobius::py::pynone ();
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief reset_tag method
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_reset_tag (io_folder_o *self, PyObject *args)
+{
+  // parse input args
+  std::string arg_tag;
+
+  try
+    {
+      arg_tag = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
+
+  // execute C++ code
+  PyObject *ret = nullptr;
+
+  try
+    {
+      self->obj->reset_tag (arg_tag);
+      ret = mobius::py::pynone ();
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief has_tag method
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_has_tag (io_folder_o *self, PyObject *args)
+{
+  // parse input args
+  std::string arg_tag;
+
+  try
+    {
+      arg_tag = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
+
+  // execute C++ code
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pybool_from_bool (self->obj->has_tag (arg_tag));
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief get_tags method
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_get_tags (io_folder_o *self, PyObject *)
+{
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pylist_from_cpp_container (
+               self->obj->get_tags (),
+               mobius::py::pystring_from_std_string
+            );
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>has_metadata</i> method implementation
+//! \param self Object
+//! \param args Argument list
+//! \return true/false
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_has_metadata (io_folder_o *self, PyObject *args)
+{
+  // Parse input args
+  std::string arg_name;
+
+  try
+    {
+      arg_name = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
+
+  // Execute C++ function
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pybool_from_bool (self->obj->has_metadata (arg_name));
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  // Return value
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>get_metadata</i> method implementation
+//! \param self Object
+//! \param args Argument list
+//! \return Metadata value
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_get_metadata (io_folder_o *self, PyObject *args)
+{
+  // Parse input args
+  std::string arg_name;
+
+  try
+    {
+      if (mobius::py::get_arg_size (args) > 0)
+        arg_name = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
+
+  // Execute C++ function
+  PyObject *ret = nullptr;
+
+  try
+    {
+      if (arg_name.empty ())
+        ret = PyMobius_Pod_Map_to_python (self->obj->get_metadata ());
+
+      else
+        ret = PyMobius_Pod_Data_from_cpp (self->obj->get_metadata (arg_name));
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  // Return value
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>set_metadata</i> method implementation
+//! \param self Object
+//! \param args Argument list
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_set_metadata (io_folder_o *self, PyObject *args)
+{
+  // Parse input args
+  std::string arg_name;
+  mobius::pod::data arg_value;
+
+  try
+    {
+      arg_name = mobius::py::get_arg_as_std_string (args, 0);
+      arg_value = PyMobius_Pod_Data_as_cpp (mobius::py::get_arg (args, 1));
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
+
+  // Execute C++ function
+  try
+    {
+      self->obj->set_metadata (arg_name, arg_value);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+      return nullptr;
+    }
+
+  // return None
+  return mobius::py::pynone ();
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>remove_metadata</i> method implementation
+//! \param self Object
+//! \param args Argument list
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_remove_metadata (io_folder_o *self, PyObject *args)
+{
+  // Parse input args
+  std::string arg_name;
+
+  try
+    {
+      arg_name = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
+
+  // Execute C++ function
+  try
+    {
+      self->obj->remove_metadata (arg_name);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+      return nullptr;
+    }
+
+  // return None
+  return mobius::py::pynone ();
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>get_streams</i> method implementation
+//! \param self Object
+//! \param args Argument list
+//! \return Streams
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_get_streams (io_folder_o *self, PyObject *)
+{
+  // Execute C++ function
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pylist_from_cpp_container (
+               self->obj->get_streams (),
+               PyMobius_Io_Stream_from_cpp
+            );
+
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  // Return value
+  return ret;
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Methods structure
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -896,31 +1449,43 @@ static PyMethodDef tp_methods[] =
     (char *) "exists",
     (PyCFunction) tp_f_exists,
     METH_VARARGS,
-    "Check if file exists"
+    "Check if folder exists"
   },
   {
     (char *) "is_deleted",
     (PyCFunction) tp_f_is_deleted,
     METH_VARARGS,
-    "Check if file is deleted"
+    "Check if folder is deleted"
   },
   {
     (char *) "is_reallocated",
     (PyCFunction) tp_f_is_reallocated,
     METH_VARARGS,
-    "Check if file is reallocated"
+    "Check if folder is reallocated"
   },
   {
     (char *) "is_hidden",
     (PyCFunction) tp_f_is_hidden,
     METH_VARARGS,
-    "Check if file is hidden"
+    "Check if folder is hidden"
   },
   {
     (char *) "is_browseable",
     (PyCFunction) tp_f_is_browseable,
     METH_VARARGS,
-    "Check if file is browseable"
+    "Check if folder is browseable"
+  },
+  {
+    (char *) "is_folder",
+    (PyCFunction) tp_f_is_folder,
+    METH_VARARGS,
+    "Check if entry is a folder"
+  },
+  {
+    (char *) "is_file",
+    (PyCFunction) tp_f_is_file,
+    METH_VARARGS,
+    "Check if entry is a file"
   },
   {
     (char *) "get_extension",
@@ -940,6 +1505,24 @@ static PyMethodDef tp_methods[] =
     METH_VARARGS,
     "Get children"
   },
+  {
+    (char *) "get_child_by_name",
+    (PyCFunction) tp_f_get_child_by_name,
+    METH_VARARGS,
+    "Get child by name"
+  },
+  {
+    (char *) "get_child_by_path",
+    (PyCFunction) tp_f_get_child_by_path,
+    METH_VARARGS,
+    "Get child by path"
+  },
+  {
+    (char *) "get_children_by_name",
+    (PyCFunction) tp_f_get_children_by_name,
+    METH_VARARGS,
+    "Get children by name"
+  },
   {
     (char *) "new_file",
     (PyCFunction) tp_f_new_file,
@@ -994,6 +1577,60 @@ static PyMethodDef tp_methods[] =
     METH_VARARGS,
     "Rename folder"
   },
+  {
+    (char *) "set_tag",
+    (PyCFunction) tp_f_set_tag,
+    METH_VARARGS,
+    "Set tag"
+  },
+  {
+    (char *) "reset_tag",
+    (PyCFunction) tp_f_reset_tag,
+    METH_VARARGS,
+    "Reset tag"
+  },
+  {
+    (char *) "has_tag",
+    (PyCFunction) tp_f_has_tag,
+    METH_VARARGS,
+    "Check if tag is set"
+  },
+  {
+    (char *) "get_tags",
+    (PyCFunction) tp_f_get_tags,
+    METH_VARARGS,
+    "Get tags"
+  },
+  {
+    (char *) "has_metadata",
+    (PyCFunction) tp_f_has_metadata,
+    METH_VARARGS,
+    "Check if metadata exists"
+  },
+  {
+    (char *) "get_metadata",
+    (PyCFunction) tp_f_get_metadata,
+    METH_VARARGS,
+    "Get metadata value"
+  },
+  {
+    (char *) "set_metadata",
+    (PyCFunction) tp_f_set_metadata,
+    METH_VARARGS,
+    "Set metadata"
+  },
+  {
+    (char *) "remove_metadata",
+    (PyCFunction) tp_f_remove_metadata,
+    METH_VARARGS,
+    "Remove metadata"
+  },
+  {
+    (char *) "get_streams",
+    (PyCFunction) tp_f_get_streams,
+    METH_VARARGS,
+    "Get streams"
+  },
   {NULL, NULL, 0, NULL} // sentinel
 };
 
