@@ -1,6 +1,6 @@
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # Mobius Forensic Toolkit
-# Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019 Eduardo Aguiar
+# Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021 Eduardo Aguiar
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -150,8 +150,8 @@ class SAM_V (object):
     [120, 'workstation_offset', 'uint32_le', 'workstation field offset'],
     [124, 'workstation_size', 'uint32_le', 'workstation field size in bytes'],
     [128, 'dummy14', 'uint32_le', '???'],
-    [132, 'dummy15_offset', 'uint32_le', '??? dummy15 field offset'],
-    [136, 'dummy15_size', 'uint32_le', 'dummy15 field size in bytes'],
+    [132, 'hours_allowed_offset', 'uint32_le', 'Hours allowed offset'],
+    [136, 'hours_allowed_size', 'uint32_le', 'Hours allowed size'],
     [140, 'dummy16', 'uint32_le', '???'],
     [144, 'dummy17_offset', 'uint32_le', '??? dummy17 field offset'],
     [148, 'dummy17_size', 'uint32_le', 'dummy17 field size in bytes'],
@@ -171,7 +171,7 @@ class SAM_V (object):
   ]
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  # @brief initialize object
+  # @brief Initialize object
   # @param data V value data
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def __init__ (self, data):
@@ -215,8 +215,8 @@ class SAM_V (object):
     workstation_offset,         \
     workstation_size,           \
     dummy14,                    \
-    dummy15_offset,             \
-    dummy15_size,               \
+    hours_allowed_offset,       \
+    hours_allowed_size,         \
     self.dummy16,               \
     dummy17_offset,             \
     dummy17_size,               \
@@ -235,7 +235,7 @@ class SAM_V (object):
     dummy24 = struct.unpack ("<IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII", data[:204])
 
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-    # parse data fields
+    # Parse data fields
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
     self.username = self.__get_text (data, name_offset, name_size)
     self.fullname = self.__get_text (data, fullname_offset, fullname_size)
@@ -247,22 +247,22 @@ class SAM_V (object):
     self.logon_script = self.__get_text (data, logon_script_offset, logon_script_size)
     self.profile_path = self.__get_text (data, profile_path_offset, profile_path_size)
     self.workstation = self.__get_text (data, workstation_offset, workstation_size)
-    self.dummy15 = dump (self.__get_field (data, dummy15_offset, dummy15_size))
+    self.hours_allowed = self.__get_field (data, hours_allowed_offset, hours_allowed_size)
     self.dummy17 = dump (self.__get_field (data, dummy17_offset, dummy17_size))
 
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-    # hashes
+    # Hashes
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
     self.lm_hash = self.__get_field (data, lm_hash_offset, lm_hash_size)
     self.nt_hash = self.__get_field (data, nt_hash_offset, nt_hash_size)
-    self.lm_hash_history = self.__get_field (data, lm_hash_history_offset, lm_hash_history_size)
     self.nt_hash_history = self.__get_field (data, nt_hash_history_offset, nt_hash_history_size)
+    self.lm_hash_history = self.__get_field (data, lm_hash_history_offset, lm_hash_history_size)
 
     # account is valid
     self.__is_valid = True
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  # @brief get UTF-16 field as UTF-8 text
+  # @brief Get UTF-16 field as UTF-8 text
   # @param data, offset, size
   # @return text
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -394,7 +394,7 @@ class TS_Property (object):
     self.__is_valid = True
     
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  # @brief check if object is valid
+  # @brief Check if object is valid
   # @return true/false
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def __nonzero__ (self):
@@ -402,7 +402,7 @@ class TS_Property (object):
 
 
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-# @brief generate SAM cryptographic key
+# @brief Ggenerate SAM cryptographic key
 # @param registry registry object
 # @return SAM key or None
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -427,35 +427,36 @@ def get_sam_key (registry):
       md5.update (syskey)
       md5.update ('0123456789012345678901234567890123456789\0')
       
-      rc4 = mobius.crypt.cipher_rc4 (md5.get_digest ())
+      rc4 = mobius.crypt.cipher ("rc4", md5.get_digest ())
       sam_key = rc4.encrypt (f[0x80:0xa0])[:16]
     
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
     # Revision 0x10003 (Win10)
-    # @see https://github.com/gentilkiwi/mimikatz/wiki
+    # @see https://github.com/gentilkiwi/mimikatz/wiki (kuhl_m_lsadump_getSamKey)
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
     elif revision == 0x10003:
       struct_revision, struct_size, check_len, data_len, salt = struct.unpack ('<IIII16s', f[0x68:0x88])
       data = f[0x88:0x88 + data_len]            # followed by check
       
-      aes = mobius.crypt.cipher_aes (syskey, "cbc", salt)
+      aes = mobius.crypt.cipher ("aes", syskey, "cbc", salt)
       sam_key = aes.decrypt (data)[:16]         # AES-128 (16 bytes key)
 
     else:
-      print 'Domains\\Account\\F unknown revision: %08x' % revision
+      mobius.core.logf ('Domains\\Account\\F unknown revision: %08x' % revision)
 
   return sam_key
 
+
 class passwd_hash (object):
   pass
 
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-# @brief user account
+# @brief OS User account
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 class user_account (object):
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  # @brief initialize object
+  # @brief Initialize object
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def __init__ (self, sam_key, f, v):
     self.__sam_key = sam_key
@@ -525,20 +526,20 @@ class user_account (object):
     self.__is_valid = True
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  # @brief check if object is valid
+  # @brief Check if object is valid
   # @return true/false
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def __nonzero__ (self):
     return self.__is_valid
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  # @brief decrypt user hash
+  # @brief Decrypt user hash
   # @param hdata hash encrypted data
   # @param type (NT, LM)
   # @todo revision >= 0x20000 (AES)
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  def __decrypt_user_hash (self, data, type, is_current):
-    revision = struct.unpack ('<I', data[:4])[0]
+  def __decrypt_user_hash (self, data, htype, is_current):
+    pekid, revision = struct.unpack ('<HH', data[:4])
     
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
     # DES final key1 and key2 values
@@ -548,16 +549,16 @@ class user_account (object):
     key2 = rid_data[3] + rid_data + rid_data[:2]
 
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-    # Revision 0x1000z - Up to Win8.1
+    # Revision 0x01 - Up to Win8.1
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-    if revision < 0x20000:
-      if type == "nt":
+    if revision == 1:
+      if htype == "nt":
         if is_current:
           ntlm_str = "NTPASSWORD\0"
         else:
           ntlm_str = "NTPASSWORDHISTORY\0"
 
-      elif type == "lm":
+      elif htype == "lm":
         if is_current:
           ntlm_str = "LMPASSWORD\0"
         else:
@@ -568,26 +569,30 @@ class user_account (object):
       md5.update (rid_data)
       md5.update (ntlm_str)
       
-      rc4 = mobius.crypt.cipher_rc4 (md5.get_digest ())
+      rc4 = mobius.crypt.cipher ("rc4", md5.get_digest ())
       obfkey = rc4.decrypt (data[4:])
       
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-    # Revision 0x2000z - Win10
+    # Revision 0x02 - Win10
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-    else:
-      aes_revision, hash_size = struct.unpack ('<II', data[:8])
-      
-      if hash_size:
+    elif revision == 2:
+      aes_revision, data_size = struct.unpack ('<II', data[:8])
+
+      SALT_LENGTH = 16
+      if data_size:
         salt = data[8:24]
-        hdata = data[24:24 + hash_size]
-        aes = mobius.crypt.cipher_aes (self.__sam_key, "cbc", salt)
+        hdata = data[24:24 + data_size]
+        aes = mobius.crypt.cipher ("aes", self.__sam_key, "cbc", salt)
         obfkey = aes.decrypt (hdata)
 
       else:
         obfkey = ''
 
+    else:
+      mobius.core.logf ('SAM_HASH unknown revision: %d' % revision)
+
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-    # get hashes from obfkey
+    # Get hashes from obfkey
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
     NTLM_HASH_SIZE = 16
     hashes = []
@@ -595,11 +600,12 @@ class user_account (object):
     for i in xrange (0, len (obfkey), NTLM_HASH_SIZE):
       h = passwd_hash ()
       h.revision = revision
-      h.type = type
+      h.pekid = pekid
+      h.type = htype
       h.is_current = is_current
 
-      des1 = mobius.crypt.cipher_des (key1)
-      des2 = mobius.crypt.cipher_des (key2)
+      des1 = mobius.crypt.cipher ("des", key1)
+      des2 = mobius.crypt.cipher ("des", key2)
       h.value = des1.decrypt (obfkey[i:i + 8]) + des2.decrypt (obfkey[i + 8:i + NTLM_HASH_SIZE])
       hashes.append (h)
 
